@@ -1,8 +1,8 @@
 /* internals.h: functions which need to be called inter-file by libspectrum
                 routines, but not by user code
-   Copyright (c) 2001-2008 Philip Kendall, Darren Salt
+   Copyright (c) 2001-2015 Philip Kendall, Darren Salt
 
-   $Id: internals.h 4695 2012-05-07 02:03:10Z fredm $
+   $Id: internals.h 5422 2016-04-29 12:54:59Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,6 +51,37 @@
 #endif				/* _MSC_VER > 1200 */
 #endif				/* #ifdef _MSC_VER */
 
+#if defined(__GNUC__) && defined(__GNUC_MINOR__)
+  #define GNUC_VERSION \
+    (__GNUC__ << 16) + __GNUC_MINOR__
+  #define GNUC_PREREQ(maj, min) \
+    (GNUC_VERSION >= ((maj) << 16) + (min))
+#else
+  #define GNUC_PREREQ(maj, min) 0
+#endif
+
+#define BUILD_BUG_ON_ZERO(e) \
+  (sizeof(struct { int:-!!(e) * 1234; }))
+
+#if !GNUC_PREREQ(3, 1) || defined( __STRICT_ANSI__ )
+  #define MUST_BE_ARRAY(a) \
+    BUILD_BUG_ON_ZERO(sizeof(a) % sizeof(*a))
+#else
+  #define SAME_TYPE(a, b) \
+    __builtin_types_compatible_p(typeof(a), typeof(b))
+  #define MUST_BE_ARRAY(a) \
+    BUILD_BUG_ON_ZERO(SAME_TYPE((a), &(*a)))
+#endif
+
+#define ARRAY_SIZE(a) ( \
+  (sizeof(a) / sizeof(*a)) \
+   + MUST_BE_ARRAY(a))
+
+/* C90 lacks SIZE_MAX.  size_t is always unsigned so this is safe. */
+#ifndef SIZE_MAX
+#define SIZE_MAX (~(size_t)0)
+#endif
+
 /* VC6 lacks M_LN2, and VS2003+ require _USE_MATH_DEFINES defined before math.h
  */
 #ifndef M_LN2
@@ -104,7 +135,7 @@ libspectrum_bzip2_inflate( const libspectrum_byte *bzptr, size_t bzlength,
 
 /* The TZX file signature */
 
-extern const char *libspectrum_tzx_signature;
+extern const char * const libspectrum_tzx_signature;
 
 /* Convert a 48K memory dump into separate RAM pages */
 
