@@ -1,7 +1,7 @@
 /* make-perl.c: Generate a perl script to create the libspectrum_* typedefs
    Copyright (c) 2002-2003,2015 Philip Kendall, Darren Salt
 
-   $Id: make-perl.c 5422 2016-04-29 12:54:59Z fredm $
+   $Id: make-perl.c 5676 2016-07-09 13:56:51Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -260,10 +260,13 @@ int main(void)
 "\n"
 "WIN32_DLL GArray* g_array_new( gboolean zero_terminated, gboolean clear,\n"
 "		      guint element_size );\n"
+"WIN32_DLL GArray* g_array_sized_new( gboolean zero_terminated, gboolean clear,\n"
+"                   guint element_size, guint reserved_size );\n"
 "#define g_array_append_val(a,v) g_array_append_vals( a, &(v), 1 );\n"
 "WIN32_DLL GArray* g_array_append_vals( GArray *array, gconstpointer data, guint len );\n"
 "#define g_array_index(a,t,i) (*(((t*)a->data)+i))\n"
 "WIN32_DLL GArray* g_array_set_size( GArray *array, guint length );\n"
+"WIN32_DLL GArray* g_array_remove_index_fast( GArray *array, guint index );\n"
 "WIN32_DLL gchar* g_array_free( GArray *array, gboolean free_segment );\n"
 "\n" );
   if( sizeof( void* ) == sizeof( int ) ) {
