@@ -1,7 +1,8 @@
 /* memory.c: memory allocator routines
    Copyright (c) 2008 Philip Kendall
+   Copyright (c) 2015 Stuart Brady
 
-   $Id: memory.c 4774 2012-11-26 22:57:29Z sbaldovi $
+   $Id: memory.c 5438 2016-05-01 16:27:08Z sbaldovi $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +30,12 @@
 
 #include "internals.h"
 
+/* We don't want the macro from libspectrum.h */
+#undef libspectrum_calloc
+
+/* Deprecated, removed from header in favour of macro. */
+void* libspectrum_calloc( size_t nmemb, size_t size );
+
 libspectrum_malloc_fn_t libspectrum_malloc_fn = malloc;
 libspectrum_calloc_fn_t libspectrum_calloc_fn = calloc;
 libspectrum_realloc_fn_t libspectrum_realloc_fn = realloc;
@@ -46,9 +53,21 @@ libspectrum_malloc( size_t size )
 }
 
 void*
-libspectrum_calloc( size_t nmemb, size_t size )
+libspectrum_malloc_n( size_t nmemb, size_t size )
 {
-  void *ptr = libspectrum_calloc_fn( nmemb, size );
+  if( nmemb > SIZE_MAX / size ) abort();
+
+  return libspectrum_malloc( nmemb * size );
+}
+
+void*
+libspectrum_malloc0_n( size_t nmemb, size_t size )
+{
+  void *ptr;
+
+  if( nmemb > SIZE_MAX / size ) abort();
+
+  ptr = libspectrum_calloc_fn( nmemb, size );
 
   /* If nmemb * size == 0, acceptable to return NULL */
   if( ( nmemb * size ) && !ptr ) abort();
@@ -56,6 +75,12 @@ libspectrum_calloc( size_t nmemb, size_t size )
   return ptr;
 }
 
+void*
+libspectrum_calloc( size_t nmemb, size_t size )
+{
+  return libspectrum_malloc0_n( nmemb, size );
+}
+
 void*
 libspectrum_realloc( void *ptr, size_t size )
 {
@@ -67,6 +92,14 @@ libspectrum_realloc( void *ptr, size_t size )
   return ptr;
 }
 
+void*
+libspectrum_realloc_n( void *ptr, size_t nmemb, size_t size )
+{
+  if( nmemb > SIZE_MAX / size ) abort();
+
+  return libspectrum_realloc( ptr, nmemb * size );
+}
+
 void
 libspectrum_free( void *ptr )
 {
@@ -82,6 +115,7 @@ libspectrum_mem_set_vtable( libspectrum_mem_vtable_t *table )
   libspectrum_free_fn = table->free;
 
 #ifdef HAVE_LIB_GLIB
+#if !GLIB_CHECK_VERSION( 2, 44, 0 )
   {
     GMemVTable glib_table;
 
@@ -91,5 +125,6 @@ libspectrum_mem_set_vtable( libspectrum_mem_vtable_t *table )
 
     g_mem_set_vtable( &glib_table );
   }
+#endif                  /* #if !GLIB_CHECK_VERSION( 2, 44, 0 ) */
 #endif                  /* #ifdef HAVE_LIB_GLIB */
 }
