@@ -1,7 +1,7 @@
 /* rzx.c: routines for dealing with .rzx files
-   Copyright (c) 2002-2008 Philip Kendall
+   Copyright (c) 2002-2015 Philip Kendall
 
-   $Id: rzx.c 4780 2012-11-29 19:09:52Z sbaldovi $
+   $Id: rzx.c 5422 2016-04-29 12:54:59Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ typedef struct input_block_t {
 
   /* Used for recording to note the last non-repeated frame. We can't
      really use a direct pointer to the frame here as that will move
-     around when we do a realloc on the array, so just dereference it
+     around when we do a renew on the array, so just dereference it
      every time */
   size_t non_repeat;
 
@@ -173,7 +173,7 @@ rzx_write_signed_end( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		      libspectrum_rzx_dsa_key *key );
 
 /* The signature used to identify .rzx files */
-const char *rzx_signature = "RZX!";
+static const char * const rzx_signature = "RZX!";
 
 /* The IN count used to signify 'repeat last frame' */
 const libspectrum_word libspectrum_rzx_repeat_frame = 0xffff;
@@ -185,7 +185,7 @@ const libspectrum_word libspectrum_rzx_repeat_frame = 0xffff;
 static void
 block_alloc( rzx_block_t **block, libspectrum_rzx_block_id type )
 {
-  *block = libspectrum_malloc( sizeof( **block ) );
+  *block = libspectrum_new( rzx_block_t, 1 );
   (*block)->type = type;
 }
 
@@ -260,7 +260,7 @@ find_block( gconstpointer a, gconstpointer b )
 libspectrum_rzx*
 libspectrum_rzx_alloc( void )
 {
-  libspectrum_rzx *rzx = libspectrum_malloc( sizeof( *rzx ) );
+  libspectrum_rzx *rzx = libspectrum_new( libspectrum_rzx, 1 );
   rzx->blocks = NULL;
   rzx->current_block = NULL;
   rzx->current_input = NULL;
@@ -281,6 +281,7 @@ libspectrum_rzx_start_input( libspectrum_rzx *rzx, libspectrum_dword tstates )
   rzx->current_input->frames = NULL;
   rzx->current_input->allocated = 0;
   rzx->current_input->count = 0;
+  rzx->current_input->non_repeat = 0;
 
   rzx->blocks = g_slist_append( rzx->blocks, block );
 }
@@ -393,12 +394,37 @@ libspectrum_rzx_rollback_to( libspectrum_rzx *rzx, libspectrum_snap **snap,
   return LIBSPECTRUM_ERROR_NONE;
 }
 
+static libspectrum_error
+input_block_resize( input_block_t *input, size_t new_count )
+{
+  libspectrum_rzx_frame_t *ptr;
+  size_t new_allocated;
+
+  /* Get more space if we need it; allocate twice as much as we currently
+     have, with a minimum of 50 */
+  if( new_count > input->allocated ) {
+
+    new_allocated = input->allocated >= 25 ? 2 * input->allocated : 50;
+    if( new_allocated < new_count ) new_allocated = new_count;
+
+    ptr = libspectrum_renew( libspectrum_rzx_frame_t, input->frames,
+                             new_allocated );
+    if( !ptr ) return LIBSPECTRUM_ERROR_MEMORY;
+
+    input->frames = ptr;
+    input->allocated = new_allocated;    
+  }
+
+  return LIBSPECTRUM_ERROR_NONE;
+}
+
 libspectrum_error
 libspectrum_rzx_store_frame( libspectrum_rzx *rzx, size_t instructions,
 			     size_t count, libspectrum_byte *in_bytes )
 {
   input_block_t *input;
   libspectrum_rzx_frame_t *frame;
+  libspectrum_error error;
 
   input = rzx->current_input;
 
@@ -411,19 +437,10 @@ libspectrum_rzx_store_frame( libspectrum_rzx *rzx, size_t instructions,
     return LIBSPECTRUM_ERROR_INVALID;
   }
 
-  /* Get more space if we need it; allocate twice as much as we currently
-     have, with a minimum of 50 */
-  if( input->count == input->allocated ) {
-
-    libspectrum_rzx_frame_t *ptr; size_t new_allocated;
-
-    new_allocated = input->allocated >= 25 ? 2 * input->allocated : 50;
-
-    ptr = realloc( input->frames, new_allocated * sizeof( *ptr ) );
-    if( !ptr ) return LIBSPECTRUM_ERROR_MEMORY;
-
-    input->frames = ptr;
-    input->allocated = new_allocated;
+  /* Get more space if we need it */
+  if( input->allocated == input->count ) {
+    error = input_block_resize( input, input->count + 1 );
+    if( error ) return error;
   }
 
   frame = &input->frames[ input->count ];
@@ -438,6 +455,8 @@ libspectrum_rzx_store_frame( libspectrum_rzx *rzx, size_t instructions,
     ) {
 	
     frame->repeat_last = 1;
+    frame->count = 0;
+    frame->in_bytes = NULL;
 
   } else {
 
@@ -449,7 +468,7 @@ libspectrum_rzx_store_frame( libspectrum_rzx *rzx, size_t instructions,
 
     if( count ) {
 
-      frame->in_bytes = libspectrum_malloc( count * sizeof( *( frame->in_bytes ) ) );
+      frame->in_bytes = libspectrum_new( libspectrum_byte, count );
 
       memcpy( frame->in_bytes, in_bytes,
 	      count * sizeof( *( frame->in_bytes ) ) );
@@ -601,6 +620,7 @@ libspectrum_rzx_free( libspectrum_rzx *rzx )
 {
   g_slist_foreach( rzx->blocks, block_free_wrapper, NULL );
   g_slist_free( rzx->blocks );
+  libspectrum_free( rzx );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -990,7 +1010,7 @@ rzx_read_input( libspectrum_rzx *rzx,
   (*ptr)++;
 
   /* Allocate memory for the frames */
-  block->frames = libspectrum_malloc( block->count * sizeof( *block->frames ) );
+  block->frames = libspectrum_new( libspectrum_rzx_frame_t, block->count );
   block->allocated = block->count;
 
   /* Fetch the T-state counter and the flags */
@@ -1093,7 +1113,7 @@ rzx_read_frames( input_block_t *block, const libspectrum_byte **ptr,
     if( block->frames[i].count ) {
 
       block->frames[i].in_bytes =
-	libspectrum_malloc( block->frames[i].count * sizeof( libspectrum_byte ) );
+	libspectrum_new( libspectrum_byte, block->frames[i].count );
       memcpy( block->frames[i].in_bytes, *ptr, block->frames[i].count );
 
     } else {
@@ -1264,6 +1284,9 @@ libspectrum_rzx_write( libspectrum_byte **buffer, size_t *length,
       break;
 
     case LIBSPECTRUM_RZX_INPUT_BLOCK:
+      /* z80 snapshots can't safely store an intermediate state */
+      snap_format = LIBSPECTRUM_ID_SNAPSHOT_SZX;
+
       error = rzx_write_input( &( block->types.input ), buffer, &ptr, length,
 			       compress );
       if( error != LIBSPECTRUM_ERROR_NONE ) return error;
@@ -1621,7 +1644,13 @@ void
 libspectrum_rzx_insert_snap( libspectrum_rzx *rzx, libspectrum_snap *snap,
 			     int where )
 {
-  rzx->blocks = g_slist_insert( rzx->blocks, snap, where );
+  rzx_block_t *block;
+
+  block_alloc( &block, LIBSPECTRUM_RZX_SNAPSHOT_BLOCK );
+  block->types.snap.snap = snap;
+  block->types.snap.automatic = 0;
+
+  rzx->blocks = g_slist_insert( rzx->blocks, block, where );
 }
 
 /*
@@ -1640,6 +1669,13 @@ libspectrum_rzx_iterator_next( libspectrum_rzx_iterator it )
   return it->next;
 }
 
+libspectrum_rzx_iterator
+libspectrum_rzx_iterator_last( libspectrum_rzx *rzx )
+{
+  /* This function iterates over the whole list */
+  return g_slist_last( rzx->blocks );
+}
+
 libspectrum_rzx_block_id
 libspectrum_rzx_iterator_get_type( libspectrum_rzx_iterator it )
 {
@@ -1686,3 +1722,86 @@ libspectrum_rzx_iterator_snap_is_automatic( libspectrum_rzx_iterator it )
 
   return block->types.snap.automatic;
 }
+
+static libspectrum_error
+input_block_merge( input_block_t *input, input_block_t *next_input )
+{
+  libspectrum_error error;
+
+  /* Get more space if we need it */
+  if( input->allocated < input->count + next_input->count ) {
+    error = input_block_resize( input, input->count + next_input->count );
+    if( error ) return error;
+  }
+
+  /* Note in_bytes are not duplicated */
+  memcpy( &( input->frames[input->count] ), next_input->frames,
+          next_input->count * sizeof( libspectrum_rzx_frame_t ) );
+
+  input->non_repeat = input->count + next_input->non_repeat;
+  input->count += next_input->count;
+  next_input->count = 0; /* don't free reused in_bytes */
+
+  return 0;
+}
+
+libspectrum_error
+libspectrum_rzx_finalise( libspectrum_rzx *rzx )
+{
+  GSList *list, *item, *next_item;
+  rzx_block_t *block, *next_block;
+  libspectrum_error error;
+  int first_snap = 1;
+  int finalised = 0;
+
+  /* Delete interspersed snapshots */
+  list = rzx->blocks;
+
+  while( list ) {
+    item = list;
+    block = list->data;
+    list = list->next;
+
+    if( block->type == LIBSPECTRUM_RZX_SNAPSHOT_BLOCK ) {
+      if( first_snap ) {
+        first_snap = 0;
+      } else {
+        block_free( block );
+        rzx->blocks = g_slist_delete_link( rzx->blocks, item );
+        finalised = 1;
+      }
+    }
+  }
+
+  /* Merge adjacent input blocks */
+  list = rzx->blocks;
+
+  while( list ) {
+    block = list->data;
+
+    if( block->type == LIBSPECTRUM_RZX_INPUT_BLOCK ) {
+
+      next_item = list->next;
+      if( !next_item ) break;
+
+      next_block = next_item->data;
+
+      if( next_block->type == LIBSPECTRUM_RZX_INPUT_BLOCK ) {
+        error = input_block_merge( &( block->types.input ),
+                                   &( next_block->types.input ) );
+        if( error ) return error;
+
+        block_free( next_block );
+        rzx->blocks = g_slist_delete_link( rzx->blocks, next_item );
+        finalised = 1;
+      } else {
+        list = list->next;
+      }
+
+    } else {
+      list = list->next;
+    }
+  }
+
+  return finalised? LIBSPECTRUM_ERROR_NONE : LIBSPECTRUM_ERROR_INVALID;
+}
