@@ -1,7 +1,8 @@
 /* sna.c: Routines for handling .sna snapshots
    Copyright (c) 2001-2002 Philip Kendall
+   Copyright (c) 2016 Fredrick Meunier
 
-   $Id: sna.c 4836 2012-12-31 15:40:10Z zubzero $
+   $Id: sna.c 5486 2016-05-14 12:39:37Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -185,7 +186,7 @@ libspectrum_sna_read_data( const libspectrum_byte *buffer,
   case LIBSPECTRUM_MACHINE_PENT:
     
     for( i=0; i<8; i++ ) {
-      libspectrum_byte *ram = libspectrum_malloc( 0x4000 * sizeof( *ram ) );
+      libspectrum_byte *ram = libspectrum_new( libspectrum_byte, 0x4000 );
       libspectrum_snap_set_pages( snap, i, ram );
     }
 
@@ -335,6 +336,18 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
   if( libspectrum_snap_spectranet_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
+  /* We don't save the uSource state at all */
+  if( libspectrum_snap_usource_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  /* We don't save the DISCiPLE state at all */
+  if( libspectrum_snap_disciple_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  /* We don't save the Didaktik80 state at all */
+  if( libspectrum_snap_didaktik80_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
   ptr = *buffer;
 
   write_header( buffer, &ptr, length, snap );
