@@ -1,7 +1,7 @@
 /* snapshot.c: Snapshot handling routines
    Copyright (c) 2001-2009 Philip Kendall, Darren Salt
 
-   $Id: snapshot.c 4860 2013-01-26 11:33:25Z fredm $
+   $Id: snapshot.c 5486 2016-05-14 12:39:37Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -223,6 +223,41 @@ libspectrum_snap_alloc( void )
   
   libspectrum_snap_set_zx_printer_active( snap, 0 );
 
+  libspectrum_snap_set_usource_active( snap, 0 );
+  libspectrum_snap_set_usource_paged( snap, 0 );
+  libspectrum_snap_set_usource_custom_rom( snap, 0 );
+  libspectrum_snap_set_usource_rom( snap, 0, NULL );
+  libspectrum_snap_set_usource_rom_length( snap, 0, 0 );
+
+  libspectrum_snap_set_disciple_active( snap, 0 );
+  libspectrum_snap_set_disciple_paged( snap, 0 );
+  libspectrum_snap_set_disciple_inhibit_button( snap, 0 );
+  libspectrum_snap_set_disciple_drive_count( snap, 0 );
+  libspectrum_snap_set_disciple_custom_rom( snap, 0 );
+  libspectrum_snap_set_disciple_direction( snap, 0 );
+  libspectrum_snap_set_disciple_control( snap, 0 );
+  libspectrum_snap_set_disciple_track ( snap, 0 );
+  libspectrum_snap_set_disciple_sector( snap, 0 );
+  libspectrum_snap_set_disciple_data  ( snap, 0 );
+  libspectrum_snap_set_disciple_status( snap, 0 );
+  libspectrum_snap_set_disciple_rom( snap, 0, NULL );
+  libspectrum_snap_set_disciple_rom_length( snap, 0, 0 );
+  libspectrum_snap_set_disciple_ram( snap, 0, NULL );
+
+  libspectrum_snap_set_didaktik80_active( snap, 0 );
+  libspectrum_snap_set_didaktik80_paged( snap, 0 );
+  libspectrum_snap_set_didaktik80_drive_count( snap, 0 );
+  libspectrum_snap_set_didaktik80_custom_rom( snap, 0 );
+  libspectrum_snap_set_didaktik80_direction( snap, 0 );
+  libspectrum_snap_set_didaktik80_aux( snap, 0 );
+  libspectrum_snap_set_didaktik80_track ( snap, 0 );
+  libspectrum_snap_set_didaktik80_sector( snap, 0 );
+  libspectrum_snap_set_didaktik80_data  ( snap, 0 );
+  libspectrum_snap_set_didaktik80_status( snap, 0 );
+  libspectrum_snap_set_didaktik80_rom( snap, 0, NULL );
+  libspectrum_snap_set_didaktik80_rom_length( snap, 0, 0 );
+  libspectrum_snap_set_didaktik80_ram( snap, 0, NULL );
+
   return snap;
 }
 
@@ -271,6 +306,19 @@ libspectrum_snap_free( libspectrum_snap *snap )
   if( libspectrum_snap_spectranet_ram( snap, 0 ) )
     libspectrum_free( libspectrum_snap_spectranet_ram( snap, 0 ) );
 
+  if( libspectrum_snap_usource_rom( snap, 0 ) )
+    libspectrum_free( libspectrum_snap_usource_rom( snap, 0 ) );
+
+  if( libspectrum_snap_disciple_rom( snap, 0 ) )
+    libspectrum_free( libspectrum_snap_disciple_rom( snap, 0 ) );
+  if( libspectrum_snap_disciple_ram( snap, 0 ) )
+    libspectrum_free( libspectrum_snap_disciple_ram( snap, 0 ) );
+
+  if( libspectrum_snap_didaktik80_rom( snap, 0 ) )
+    libspectrum_free( libspectrum_snap_didaktik80_rom( snap, 0 ) );
+  if( libspectrum_snap_didaktik80_ram( snap, 0 ) )
+    libspectrum_free( libspectrum_snap_didaktik80_ram( snap, 0 ) );
+
   libspectrum_free( snap );
 
   return LIBSPECTRUM_ERROR_NONE;
@@ -424,7 +472,7 @@ libspectrum_split_to_48k_pages( libspectrum_snap *snap,
   }
 
   for( i = 0; i < 3; i++ )
-    buffer[i] = libspectrum_malloc( 0x4000 * sizeof( libspectrum_byte ) );
+    buffer[i] = libspectrum_new( libspectrum_byte, 0x4000 );
 
   libspectrum_snap_set_pages( snap, 5, buffer[0] );
   libspectrum_snap_set_pages( snap, 2, buffer[1] );
