@@ -1,7 +1,7 @@
 /* tape_block.c: one tape block
-   Copyright (c) 2003-2008 Philip Kendall
+   Copyright (c) 2003-2016 Philip Kendall
 
-   $Id: tape_block.c 4433 2011-05-14 05:44:47Z fredm $
+   $Id: tape_block.c 5422 2016-04-29 12:54:59Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ data_block_init( libspectrum_tape_data_block *block,
 libspectrum_tape_block*
 libspectrum_tape_block_alloc( libspectrum_tape_type type )
 {
-  libspectrum_tape_block *block = libspectrum_malloc( sizeof( *block ) );
+  libspectrum_tape_block *block = libspectrum_new( libspectrum_tape_block, 1 );
   libspectrum_tape_block_set_type( block, type );
   return block;
 }
@@ -348,12 +348,20 @@ static libspectrum_error
 generalised_data_init( libspectrum_tape_generalised_data_block *block,
                        libspectrum_tape_generalised_data_block_state *state )
 {
-  state->state = LIBSPECTRUM_TAPE_STATE_PILOT;
+  state->state = block->pilot_table.symbols_in_block ?
+    LIBSPECTRUM_TAPE_STATE_PILOT : LIBSPECTRUM_TAPE_STATE_DATA1;
 
   state->run = 0;
   state->symbols_through_run = 0;
   state->edges_through_symbol = 0;
 
+  state->current_symbol = 0;
+  state->symbols_through_stream = 0;
+
+  state->current_byte = 0;
+  state->bits_through_byte = 0;
+  state->bytes_through_stream = 0;
+
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -381,7 +389,9 @@ data_block_init( libspectrum_tape_data_block *block,
     }
   }
 
-  state->level = block->initial_level;
+  if( block->initial_level != -1 ) {
+    state->level = block->initial_level;
+  }
 
   /* We're just before the start of the data */
   state->bytes_through_block = -1; state->bits_through_byte = 7;
@@ -445,7 +455,8 @@ libspectrum_tape_block_read_symbol_table_parameters(
   table->max_pulses = (*ptr)[0];
 
   table->symbols_in_table = (*ptr)[1];
-  if( !table->symbols_in_table ) table->symbols_in_table = 256;
+  if( !table->symbols_in_table && table->symbols_in_block ) 
+    table->symbols_in_table = 256;
 
   (*ptr) += 2;
 
@@ -470,13 +481,15 @@ libspectrum_tape_block_read_symbol_table(
     }
 
 
-    table->symbols = libspectrum_malloc( table->symbols_in_table * sizeof( *table->symbols ) );
+    table->symbols =
+      libspectrum_new( libspectrum_tape_generalised_data_symbol,
+                       table->symbols_in_table );
 
     for( i = 0, symbol = table->symbols;
 	 i < table->symbols_in_table;
 	 i++, symbol++ ) {
       symbol->edge_type = **ptr; (*ptr)++;
-      symbol->lengths = libspectrum_malloc( table->max_pulses * sizeof( *symbol->lengths ) );
+      symbol->lengths = libspectrum_new( libspectrum_word, table->max_pulses );
       for( j = 0; j < table->max_pulses; j++ ) {
 	symbol->lengths[ j ] = (*ptr)[0] + (*ptr)[1] * 0x100;
 	(*ptr) += 2;
