@@ -1,8 +1,9 @@
 /* z80em.c: Routines for handling Z80Em raw audio files
+   Copyright (c) 2015 Stuart Brady
    Copyright (c) 2002 Darren Salt
    Based on tap.c, copyright (c) 2001 Philip Kendall
 
-   $Id: z80em.c 3708 2008-07-01 08:07:01Z pak21 $
+   $Id: z80em.c 5422 2016-04-29 12:54:59Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,7 +64,7 @@ libspectrum_z80em_read( libspectrum_tape *tape,
 
   /* Claim memory for the data (it's one big lump) */
   z80em_block->length = length;
-  z80em_block->data = libspectrum_malloc( length );
+  z80em_block->data = libspectrum_new( libspectrum_byte, length );
 
   /* Copy the data across */
   memcpy( z80em_block->data, buffer, length );
